;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xc6c2594495934c89a7d5bcdbaf1c2c10 . "./code.cg2js/cg-acache.fasl")
(#xed0053b286dbbba4f9aba8ae32a05d5e . "./code.cg2js/cg-base.fasl")
(#x9b06e126cbf43ac47ce3d89b5040aadf . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x96cae90ee7c2445dcd67eea178e2f426 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x0fab8ca88694acac8449ffddd0dd3ff4 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x24ede64ce2b80dfb6831649500bcca17 . "./code.cg2js/cg-button.fasl")
(#x84577ee74721bf79f62f8e0c23a63a5c . "./code.cg2js/cg-calendar.fasl")
(#x69debda03450d14fda0d526bd23f0ce7 . "./code.cg2js/cg-caret.fasl")
(#x619e6c7d1b48e1eb502b10e12047641f . "./code.cg2js/cg-chart-or-plot.fasl")
(#x24099fcc1d92843adeed7108b8b55a1c . "./code.cg2js/cg-chart-widget.fasl")
(#xfaa2ffef35536b889647afd94f72796c . "./code.cg2js/cg-check-box.fasl")
(#xc1b011bc4aa89424ed8d6670dddef9f2 . "./code.cg2js/cg-choice-list.fasl")
(#x902b2bcec0dbe283e45a150cc214ab89 . "./code.cg2js/cg-choose-printer.fasl")
(#xc04fb7e65185ce12a25ff3c6be08df05 . "./code.cg2js/cg-class-grid.fasl")
(#x24914693fb9089b9eeb3be2300927b5d . "./code.cg2js/cg-class-slot-grid.fasl")
(#x490b5fd565612cdc5eab58faf4d4d8b2 . "./code.cg2js/cg-class-support.fasl")
(#xba93cd4ef37054acac6e7bfc328778a7 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x4c12780e8cbd553f1a73023d9aa949d6 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xaf11e1c68adc162bbecc999a35983a5b . "./code.cg2js/cg-clipboard.fasl")
(#x782749a13c53fbde3eeb01d6fef9a27d . "./code.cg2js/cg-color-dialog.fasl")
(#xcc1ac1b0225c30d0e9479c1454882011 . "./code.cg2js/cg-combo-box.fasl")
(#x79d8abe7804e9a643ffcb4040fa840ec . "./code.cg2js/cg-common-control.fasl")
(#x37d8d19600e2934350a0eb97ca8574fb . "./code.cg2js/cg-comtab.fasl")
(#xabcb737cd8dc9b91f983d92c0e12445c . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xbb6801ba59fbf755dd3d72f34250c2c8 . "./code.cg2js/cg-curve.fasl")
(#x02593112ea64993169b94e6e8192e4b4 . "./code.cg2js/cg-dde-utils.fasl")
(#xbb92408a0f402cbc53061e540aa6cec9 . "./code.cg2js/cg-dev.fasl")
(#x0161876b16caafde333a48fa28fdce9b . "./code.cg2js/cg-dialog-item.fasl")
(#x5921c443587361a5afccd35639672afb . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x4ff7d7aef9783cd76591d7f524c5f138 . "./code.cg2js/cg-directory-dialog.fasl")
(#x8dee6d749c9de05a66fa32d19b5f4cb5 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x09ddb0a43f9e87e9ddaef3eea7c20065 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x315ed65a749599bb678d212f55e897ff . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x2c744d3603c9f5cdf2bbaf5da2ef9f8e . "./code.cg2js/cg-drawable.fasl")
(#x9d450fd8b6efd6c37226e35e9edb2be6 . "./code.cg2js/cg-dropping-outline.fasl")
(#x90c48694b64a3a34c52d791ebbf7879a . "./code.cg2js/cg-edit-in-place.fasl")
(#x9c33adfd748c8ea7b8bd6a5406d240ae . "./code.cg2js/cg-editable-text.fasl")
(#xec7faffa023063126078729399cc5731 . "./code.cg2js/cg-file-dialog.fasl")
(#xda77442bbdf88691b361be8acae738a6 . "./code.cg2js/cg-fill-texture.fasl")
(#x16344b19663163a0470444c93c711bf0 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x2c1e2da7a9dfd64484b8ac831f59daf3 . "./code.cg2js/cg-font-dialog.fasl")
(#x4ae57e1c856282c80982097920f40d60 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x096012e94bba7a899085aed0ecaf8b36 . "./code.cg2js/cg-get-pixmap.fasl")
(#xa684e0a46d21f7cbdc76ed402bdf5d51 . "./code.cg2js/cg-get-position.fasl")
(#xe642d1f5cf1c30b1f7c1d80f46864544 . "./code.cg2js/cg-graphics-context.fasl")
(#x1af75534384ce747aa6f3f2dba80fb56 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x2b56f41019a00b418d5e015360949ae3 . "./code.cg2js/cg-grid-widget.fasl")
(#xa318edb1e74f66f73f8b98d42226852c . "./code.cg2js/cg-group-box.fasl")
(#xa5099f7c22da0dcf141cbde6472b064f . "./code.cg2js/cg-header-control.fasl")
(#x3c6ec76d86bcc41f99772df022e8f3ca . "./code.cg2js/cg-hotspot.fasl")
(#xca69d35e0c9a519a479b8af0ffdbe2c1 . "./code.cg2js/cg-html-dialog.fasl")
(#xd54e9a75f03822ae719df2e55a703943 . "./code.cg2js/cg-html-widget.fasl")
(#xf969cde64c869bd624f508939f41be74 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x0f1c31a8c34ca4d70dc7c1ccca412de4 . "./code.cg2js/cg-icon.fasl")
(#xa5b853ce43a5e7e37281e49f4c5cffa9 . "./code.cg2js/cg-ie.fasl")
(#xbe9d930da47665da45224dd177191e24 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x530a0f063274c8cac22187b648cabbce . "./code.cg2js/cg-intersect.fasl")
(#x8240c3ff69ca291abf795c463cae81f8 . "./code.cg2js/cg-item-list.fasl")
(#x46d634e3a63d2d1ceec67aca3d096e1a . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x26e573643c6afc6100ca1be806e96e58 . "./code.cg2js/cg-lamp.fasl")
(#xc6f101a26291fcce167019f274489a62 . "./code.cg2js/cg-layout.fasl")
(#x53f9db55ecf07e8f352bde028a4cace0 . "./code.cg2js/cg-lettered-menu.fasl")
(#x5021f7abc692cf0206f13783ec06aae9 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xcb6ab5978b466edabe6472c81a8e1c63 . "./code.cg2js/cg-lisp-text.fasl")
(#x15614d3e7ddd30cff18d8fefec9b8b27 . "./code.cg2js/cg-lisp-widget.fasl")
(#xd43d7a0434ab7b56e2bf550631c7cfd2 . "./code.cg2js/cg-list-view.fasl")
(#xf45f64f1324920aba14f6e4fa8b0e128 . "./code.cg2js/cg-mci.fasl")
(#xbe1c5bb6e632e80b9e8007d3f7fbf073 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x430de028b07c4e7aba1b2526720d03ef . "./code.cg2js/cg-menu.fasl")
(#x1d6fb9d114ab28d8021bd79b1b3df537 . "./code.cg2js/cg-message-dialog.fasl")
(#x60bb13930be0fbfa3391ae783b587570 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x64dd8453ff661824cedc45011b1d6c35 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x029c4e5f5378a3f54c1440fe254ee363 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xfa23b52a9894a6b4f0e1a1771a60a15f . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xcb97db2367f4aecb00dd758aa2a6be5e . "./code.cg2js/cg-multi-picture-button.fasl")
(#x3b05b1a84f9462b5dafe582ee3e4f87a . "./code.cg2js/cg-nodes.fasl")
(#xe3243e553cd8684c3205afd472050a86 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x34228192df6baa6f89b7ebe8459fe0f1 . "./code.cg2js/cg-object-editor.fasl")
(#x7b2dad99403c608e31bb02a8420c0de8 . "./code.cg2js/cg-ocx.fasl")
(#x22195584d47cd750d7209b7405a3f4e9 . "./code.cg2js/cg-os-widget.fasl")
(#x87816e6968484f5fa4ec1c5a297acd4d . "./code.cg2js/cg-os-window.fasl")
(#x4ab1a27010c0df2aae7bf3f8d248fd84 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x4ca4311a1c049206cf511dd20b24b40e . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xb4e876f021c983499da884a848bf777c . "./code.cg2js/cg-outline.fasl")
(#xbccc8dade5ba2a1a9ef3a0ac7074f4d1 . "./code.cg2js/cg-palette.fasl")
(#xadd0bd8e8b8d9a3980a1c6a68d57c206 . "./code.cg2js/cg-paren-matching.fasl")
(#xd904dca72e69adadd974c2db642051b1 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x7459a3edde6393eb1c76061d3f86dd21 . "./code.cg2js/cg-picture-widget.fasl")
(#xc492a02ca9c3ff991024b6ae4c11c7e5 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xfe08f50c8f34456e56545a7e2546b252 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xb02713dbc3b1dd54b4cf4f6ed51b0f53 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x1492486d1f6f11a980d267a80eb7fbc0 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x6ad261a72ec6e22d45b8141685596af0 . "./code.cg2js/cg-pixmap.fasl")
(#x11218e563cf5e88375b3e2fe55889fd6 . "./code.cg2js/cg-plot-widget.fasl")
(#x0ae89a37207d89c0b031a790168e3b57 . "./code.cg2js/cg-printing.fasl")
(#x2e652c94da33c92ffa12ec2e98bad21e . "./code.cg2js/cg-progress-indicator.fasl")
(#xb1ba09acab59fa6d18612b98cdc90578 . "./code.cg2js/cg-project-window.fasl")
(#xa63fb6fef4d88465395c291de2a39ed5 . "./code.cg2js/cg-property.fasl")
(#x749bb81c5f3064b370bc668ecc390d66 . "./code.cg2js/cg-radio-button.fasl")
(#xf3c6ec9e8b2316a9523f3749fb7e8f8e . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x6fab14671df8bb38db5050da8bb432dc . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x7d880231d46dc6d076b880a88abf0e99 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x12e78e49bef902627bd6f5a5598afe9d . "./code.cg2js/cg-rich-edit.fasl")
(#x8a78028a02c35175557951491a130061 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x59e89c2a89961f00561f1eded3c0c860 . "./code.cg2js/cg-scaling-stream.fasl")
(#xc3ef07028907a44d76f079a03bfd4680 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x58c47be666b60f2b0ac11050a71fa5f3 . "./code.cg2js/cg-scroll-bar.fasl")
(#x6a24099ada70bfb6f5db320142742709 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x45c1e398a6118ca502a457e3c444ff10 . "./code.cg2js/cg-selected-object.fasl")
(#x42144e29c0f3fa330b8623302c35206a . "./code.cg2js/cg-shortcut-menu.fasl")
(#x1dab3adc73b08450e1ff19338d67fb12 . "./code.cg2js/cg-split-bar.fasl")
(#xb10983aec4e00d2d413d58c71a2d59e2 . "./code.cg2js/cg-static-text.fasl")
(#x9a7d986d4426809b019266ff7952008f . "./code.cg2js/cg-status-bar.fasl")
(#x6df27aac4b3103341169afc341ecf188 . "./code.cg2js/cg-string-dialog.fasl")
(#xa36b166f95e7c3cacb4d88101e17bfe5 . "./code.cg2js/cg-tab-control.fasl")
(#x0d6b96528155e21196e5537fb5899a51 . "./code.cg2js/cg-template-string.fasl")
(#x948d571800ed3b6360813d305b636da6 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x4b1b20647fa193294e252e155ac59329 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x7d998b7c5f8a860c438b1c740591659b . "./code.cg2js/cg-text-edit-pane.fasl")
(#xf266b9a46484e84f0d891a5beabacfb0 . "./code.cg2js/cg-text-or-combo.fasl")
(#x777fbef1b4987ab2717e7fef97d78954 . "./code.cg2js/cg-text-widget.fasl")
(#x770c48ac2a669a9cc28ae3333576be56 . "./code.cg2js/cg-timer.fasl")
(#xeccea5365d0f8566758c98886d1945a2 . "./code.cg2js/cg-toggling-widget.fasl")
(#x8e7e25de700e1135d4da82d314392108 . "./code.cg2js/cg-toolbar.fasl")
(#x282aed05da0026681a1975e27f75693e . "./code.cg2js/cg-tooltip.fasl")
(#x92cad2f3a1bd38a97032bad40ba5e536 . "./code.cg2js/cg-touch.fasl")
(#x0944364c657cfa25b70a00790e41b490 . "./code.cg2js/cg-trackbar.fasl")
(#xef7c5a569a9a9066a2eec8b0b64840ee . "./code.cg2js/cg-tray.fasl")
(#x1d2535455c375dd30225fc2bf333a217 . "./code.cg2js/cg-up-down-control.fasl")
(#xa044fa9ff681d559e38653f01e12a0b8 . "./code.cg2js/cg-utility-dialog.fasl")
(#xe240161a2847c2cc95ea20eca84c5a40 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x977a4d7c74b5389f216caa6c8f9fd9da . "./code.cg2js/cg-web-browser.fasl")
(#x31be3851f3c2bf60e6086f4481b635ee . "./code.cg2js/cg-wrap-string.fasl")
(#x76fba1180ff3cfdb9b2c9c967dd72347 . "./code.cg2js/cg-yes-no-list.fasl")
(#x5169f133e7ae022e8ce047865763c108 . "./code.cg2js/cg-yes-no-string.fasl")
(#x5cc4bac113eabd5427a5aea3898e9a96 . "./code.cg2js/cg.fasl")
(#x48b4d9673a680e24c658b59732ac443b . "./code.idejs/ide-acache.fasl")
(#xfd4af55224272ed529231389e09d086f . "./code.idejs/ide-apropos.fasl")
(#x6e542600ca0d316df09aaf3456a80c78 . "./code.idejs/ide-base.fasl")
(#xba3162024b1b67650e601ce4b1e33a24 . "./code.idejs/ide-class-browser.fasl")
(#x4afe32e5b682adcaa5d3208f6311136e . "./code.idejs/ide-class-grapher.fasl")
(#x5d23e55f3c0c42afda5a4e25a91afd9b . "./code.idejs/ide-clipboard.fasl")
(#x5e8adf535c16482e9ea9c7f0a43fa690 . "./code.idejs/ide-dev.fasl")
(#xc97f69cbaa4f30d2ed8ad1c4a126173b . "./code.idejs/ide-editor.fasl")
(#x9bf575535c0cbd8f89de629e3c377f01 . "./code.idejs/ide-find-definitions.fasl")
(#x3c9e8d65667b7afa85a84131cd86ed4d . "./code.idejs/ide-find-in-files.fasl")
(#x78bae2bcaf1cbee57c2430dbe1133143 . "./code.idejs/ide-form.fasl")
(#x9ded0c56598c1e77440d685c56af1311 . "./code.idejs/ide-grapher.fasl")
(#xd3310996d48670cae4cb604279c4277e . "./code.idejs/ide-inspector.fasl")
(#xc02e6ebec9826ea62753ad23144a803c . "./code.idejs/ide-intro.fasl")
(#x5d240f7b157b9267fefe0fc8e2dba061 . "./code.idejs/ide-keyshort.fasl")
(#x01c4d39974adee4990b23271f09e7433 . "./code.idejs/ide-navigator.fasl")
(#x93dce9f87c1e6938108ed67b1af15203 . "./code.idejs/ide-package-list.fasl")
(#x5c5b339dc1001cd344f382f13024b873 . "./code.idejs/ide-patch-dialog.fasl")
(#x2f41f847886d9bc673d2605d942fedea . "./code.idejs/ide-process-browser.fasl")
(#xb795dbcd0227a1e46b243262a27ffd5f . "./code.idejs/ide-project.fasl")
(#x5ba119b2cc3600c0299ba687bc31bb15 . "./code.idejs/ide-trace.fasl")
(#xc05c188bd6e7d233ac8c1ddede147d96 . "./code.idejs/ide-window-list.fasl")
(#x38ecb38d0df270921deef9ae85853a78 . "./code.idejs/ide.fasl")
(#x82407180256cdbcae17acb711a3ed09f . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
